#AUTHOR Espen S. Nygaard

from pymt import *
from pymt.ui.widgets.customgesturewidget import *
import time


class CaptureGesture(MTCustomGestureWidget):

    lastGesture = None

    def __init__(self, gdb, label, bgcolor=(.4,.4,.4, .8)):
        super(CaptureGesture, self).__init__()
        self.gdb = gdb
        self.bgcolor = bgcolor
        self.lastgesture = None
        self.lastbest = None
        self.learnMode = False
        #self.gdb.loadGesturesFromFile("testSets\\testfasit.txt",addToDB=True)
        self.globalDrawList = None

        self.testNumber = 0
        self.testMode = False
        self.statusLabel = label
        self.testFileName = "default.txt"

        self.testList = [['the letter S','s1','1','S'],
                         ['the letter S','s2','2','S'],
                         ['the letter S','s3','3','S'],
                         ['the letter S','s4','4','S'],
                         ['the letter S','s5','5','S'],
                         ['the number 5','51','1','5'],
                         ['the number 5','52','2','5'],
                         ['the number 5','53','3','5'],
                         ['the number 5','54','4','5'],
                         ['the number 5','55','5','5'],
                         ['the letter N','n1','1','N'],
                         ['the letter N','n2','2','N'],
                         ['the letter N','n3','3','N'],
                         ['the letter N','n4','4','N'],
                         ['the letter N','n5','5','N'],
                         ['a circle','circle1','1','Circle'],
                         ['a circle','circle2','2','Circle'],
                         ['a circle','circle3','3','Circle'],
                         ['a circle','circle4','4','Circle'],
                         ['a circle','circle5','5','Circle'],
                         ['a square','square1','1','Square'],
                         ['a square','square2','2','Square'],
                         ['a square','square3','3','Square'],
                         ['a square','square4','4','Square'],
                         ['a square','square5','5','Square'],
                         ['a delete symbol','delete1','1','Delete'],
                         ['a delete symbol','delete2','2','Delete'],
                         ['a delete symbol','delete3','3','Delete'],
                         ['a delete symbol','delete4','4','Delete'],
                         ['a delete symbol','delete5','5','Delete'],
                         ['the number 8','81','1','8'],
                         ['the number 8','82','2','8'],
                         ['the number 8','83','3','8'],
                         ['the number 8','84','4','8'],
                         ['the number 8','85','5','8']]




    def on_touch_down(self, touch):
        if not self.collide_point(touch.x, touch.y):
            return
        super(CaptureGesture, self).on_touch_down(touch)

    def on_touch_move(self, touch):
        if not self.collide_point(touch.x, touch.y):
            return
        super(CaptureGesture, self).on_touch_move(touch)

    def on_touch_up(self, touch):
        super(CaptureGesture, self).on_touch_up(touch)
        if not self.collide_point(touch.x, touch.y):
            return

    def on_gesture(self, gesture, touch):
        # try to find gesture from database


        self.lastGesture = gesture
        best = self.gdb.find(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'dot_product')
        dir = self.gdb.find(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'direction_levenshtein')
        cstm = self.gdb.find(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'direction_custom')
        reg = self.gdb.find(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'region_levenshtein')
        anntest = self.gdb.find(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'ann')
        hybrid = self.gdb.findBest(gesture, minscore=0.1, rotation_invariant="False", recognitionMethod = 'best')


        if not hybrid :
            pass
        else:
            print "Hybrid score is", hybrid[0],hybrid[1].label
            if self.testMode == False:
                self.statusLabel.label = hybrid[1].label


        if not anntest :
            pass
        else:
            print "ANN score is", anntest[0],anntest[1].label


        if not reg :
            pass
        else:
            print "Region score is", reg[0],reg[1].label


        if not cstm :
            pass
        else:
            print "Custom direction score is", cstm[0],cstm[1].label

        if not dir:
            pass
        else:
            print "Direction score is", dir[0],dir[1].label

        if not best:
            print 'No gesture found\nString version of your last gesture :\n', self.gdb.gesture_to_str(gesture)
        else:
            print 'Gesture found, score', best[0], ':', best[1].label,':\n', self.gdb.gesture_to_str(gesture)

        print "" #make some space 

        #check if learningMode is toogled
        if self.learnMode == True:
            gesture.label = 'NewGesture'
            gesture.id = 'new'
            self.gdb.add_gesture(gesture)



        if self.testMode == True:
            currentTest = self.testList[self.testNumber]
            gesture.label = str(currentTest[3])
            gesture.id = str(currentTest[1])
            self.gdb.saveGestureToFile(self.testFileName,gesture)
            self.testNumber = self.testNumber +1
            if self.testNumber >= len(self.testList):
                self.testNumber = 0
                self.testMode = False
                self.statusLabel.label = "Thank you! =)"
            else:
                currentTest = self.testList[self.testNumber]
                self.statusLabel.label = "Please draw " + str(currentTest[0]) + ". Try number " +  str(currentTest[2]) + "."


    def draw(self):
        # draw background
        set_color(1,1,1)
        drawRectangle(pos=self.pos, size=self.size)
        
        set_color(0,0,0)
        if self.globalDrawList != None:
            for i in range(self.testNumber*35,(self.testNumber+1)*35):
                drawLine(self.globalDrawList[i],width=3)
     

    def toogleLearnMode(self,b):
        if b.state == "normal":
            self.learnMode = False
        elif b.state == "down":
            self.learnMode = True


    def startTestsetSequence(self,label):
        self.testNumber+=1

    def testGesturesInFile(self):
       
        filenames = [   "testSets\\staale.txt",
                        "testSets\\morten.txt",
                        "testSets\\mats.txt",
                        "testSets\\trond.txt",
                        "testSets\\ulf.txt",
                        "testSets\\mtdtmats.txt",
                        "testSets\\damatilkaare.txt",
                        "testSets\\kaare.txt",
                        "testSets\\kjetil.txt",
                        "testSets\\eirik.txt",
                        "testSets\\max.txt",
                        "testSets\\marius.txt"


                        ]
       
        testSets = self.gdb.loadAllTestSets(filenames)
        self.globalDrawList = []
        for testPerson in testSets:
            gNR = 0
            sNR = 1
            for gesture in testPerson:
                label,id,g = gesture
                gNR +=1
                
                l = []
                for p in g.strokes[0].points:
                    l.append((gNR*150)-50+p.x*120)
                    l.append(1250-(sNR*150)+p.y*120)
               
                self.globalDrawList.append(l)
                if gNR == 5:
                    gNR = 0
                    sNR += 1
                if sNR == 8:
                    sNR = 1
                
class GestureUI(MTWidget):


    def __init__(self, gdb, **kwargs):
        super(GestureUI, self).__init__(**kwargs)
        self.gdb = gdb

        self.status = MTLabel(label='Status', autosize=True, font_size=16)
        self.add_widget(self.status)

        self.capture = CaptureGesture(gdb,self.status)
        self.add_widget(self.capture)

        self.title = MTLabel(label='Gesture Recognition', autosize=True, font_size=42)
        self.add_widget(self.title)

        self.learnButton = MTToggleButton(label = "learnMode", autosize=True)
        self.add_widget(self.learnButton)

        
        self.testSet = MTButton(label = "Next drawing", autosize=True)
        self.add_widget(self.testSet)

        self.exTest = MTButton(label = "Load gestures", autosize=True)
        self.add_widget(self.exTest)


        @self.testSet.event
        def on_press(*largs):
                self.capture.startTestsetSequence(self.status)


        @self.learnButton.event
        def on_press(*largs):
                self.capture.toogleLearnMode(self.learnButton)

        @self.exTest.event
        def on_press(*largs):
                self.capture.testGesturesInFile()


    def on_draw(self):
        if not self.parent:
            return
        w, h = self.parent.size
        self.capture.pos =  0,0
        self.capture.size =  w,  h

        self.testSet.pos = w-self.testSet.width,  0
        self.exTest.pos = w-self.testSet.width-self.exTest.width,  0


        super(GestureUI, self).on_draw()

def pymt_plugin_activate(root, ctx):
    gdb = GestureDatabase()

    ctx.ui = GestureUI(gdb)
    root.add_widget(ctx.ui)

def pymt_plugin_deactivate(root, ctx):
    root.remove_widget(ctx.ui)

if __name__ == '__main__':
    w = MTWindow()
    ctx = MTContext()
    pymt_plugin_activate(w, ctx)
    runTouchApp()
    pymt_plugin_deactivate(w, ctx)
